import abc
from . import StorageError as StorageError, base as base
from ..core.base import ExecutorEntity as ExecutorEntity, LoggingEntity as LoggingEntity
from ..core.formatter import FormattedEntity as FormattedEntity
from ..http import URL as URL
from ..net.web.client import Session as Session
from _typeshed import Incomplete
from abc import abstractmethod
from typing import Any, FrozenSet, Union

class StorageConnectionError(StorageError): ...
class StorageDataFormatError(StorageError): ...

class AbstractHttpStorage(FormattedEntity, LoggingEntity, base.AbstractStorageReadOnly, metaclass=abc.ABCMeta):
    session: Any
    def __init__(self, *args, **kwargs) -> None: ...
    session_params: Incomplete
    async def init(self) -> None: ...
    @abstractmethod
    async def session_factory(self, **kwargs): ...
    async def reset_session(self, **kwargs) -> None: ...
    async def stop(self) -> None: ...
    def raw_key(self, key): ...
    async def request(self, url: Union[str, URL], raise_for_status: FrozenSet[int] = ..., **kwargs): ...
    def get(self, key): ...

class RoStorage(ExecutorEntity, AbstractHttpStorage):
    def set_config(self, config) -> None: ...
    async def session_factory(self, **kwargs): ...
    async def request(self, url, **kwargs): ...

class Storage(RoStorage, base.AbstractStorageWriteOnly):
    def set(self, key: Any, value: Any, *, raise_for_status: FrozenSet[int] = ...): ...
